import unittest
from modelverse_kernel.main import ModelverseKernel
from utils import execute_until_finished, MvSWrapper

class Testbreak(unittest.TestCase):
    def setUp(self):
        self.mvs = MvSWrapper()
        self.root = self.mvs.execute("RR", [])[0]
        self.mvk = ModelverseKernel(self.root)

    def test_basic(self):
        # Create a break construct
        root = self.root
        user_root = self.mvs.execute("CN", [])[0]
        user_frame = self.mvs.execute("CN", [])[0]
        phase = self.mvs.execute("CNV", ["init"])[0]
        inst = self.mvs.execute("CNV", [{"value":"break"}])[0]
        while_inst = self.mvs.execute("CNV", ["while"])[0]

        self.mvs.execute("CD", [root, "user_1", user_root])
        self.mvs.execute("CD", [user_root, "frame", user_frame])
        self.mvs.execute("CD", [user_frame, "phase", phase])
        self.mvs.execute("CD", [user_frame, "IP", inst])
        self.mvs.execute("CD", [inst, "while", while_inst])

        # Do the invocation until it returns None
        execute_until_finished(self.mvk, self.mvs)

        # Execution of the command finished, so the MvS should be in the correct new state right now
        self.assertEquals(self.mvs.execute("RD", [root, "user_1"])[0], user_root)
        self.assertEquals(self.mvs.execute("RD", [user_root, "frame"])[0], user_frame)
        self.assertEquals(self.mvs.execute("RV", [self.mvs.execute("RD", [user_frame, "phase"])[0]])[0], "finish")
        self.assertEquals(self.mvs.execute("RD", [user_frame, "IP"])[0], while_inst)
        self.assertEquals(self.mvs.execute("RD", [inst, "while"])[0], while_inst)
